
/*
 *          |          _                     _______________________
 *          |\       _/ \_                  |                       |
 *          | \_    /_    \_                |    Alexandre Botao    |
 *          \   \__/  \__   \               |     www.botao.org     |
 *           \_    \__/  \_  \              |    55-11-8244-UNIX    |
 *             \_   _/     \ |              |  alexandre@botao.org  |
 *               \_/        \|              |_______________________|
 *                           |
 */

/*______________________________________________________________________
 |                                                                      |
 |  This file is part of 'VDT' (the Visual Disk Test) by <botao.org> ;  |
 |                                                                      |
 |  'VDT' is free software: you can redistribute it and/or modify it    |
 |  under the terms of the GNU General Public License as published by   |
 |  the Free Software Foundation, either version 3 of the License, or   |
 |  (at your option) any later version.                                 |
 |                                                                      |
 |  'VDT' is distributed in the hope that it will be useful,            |
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of      |
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                |
 |  See the GNU General Public License for more details.                |
 |                                                                      |
 |  You should have received a copy of the GNU General Public License   |
 |  along with 'VDT'.  If not, see <http://www.gnu.org/licenses/>, or   |
 |  write to the Free Software Foundation, Inc.,                        |
 |  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.            |
 |______________________________________________________________________|
 */

# ifndef _VDT_H_

# define _VDT_H_

#if defined (__HP_cc)
# if __HP_cc == 111108
#  define	NOCONST
# endif
#endif

#ifdef	NOCONST
# define	const
#endif

# include	<sys/time.h>
# include	<sys/types.h>
# include	<sys/stat.h>
# include	<sys/utsname.h>

# include	<fcntl.h>
# include	<math.h>
# include	<time.h>
# include	<ctype.h>
# include	<errno.h>
# include	<stdio.h>
# include	<stdlib.h>
# include	<unistd.h>
# include	<signal.h>
# include	<string.h>
# include	<limits.h>

# include	"bud.h"

#ifdef CYGWIN
# include	<ncurses/curses.h>
#else
# include	<curses.h>
#endif

/*	__	__	__	__	__	__	__	__	*/

# if ! ( defined (SYSIO) || defined (SIMIO) || defined (BUFIO) )
#	error	"must define {SYS|BUF|SIM}IO"
# endif

/*	__	__	__	__	__	__	__	__	*/

# define	BANNER			SWNAME

# define	VDTLOG			"vdt.log"
# define	VDTERR			"vdt.err"
# define	VDTOPT			"vdt.opt"

# define	DFLVDTDIR		"."
# define	DOTVDTDIR		".vdt"

# define	MAXPATHNAMELEN		   1024

# define	MAXVDTARG			   1024

# define	DFLSECSIZ			    512

# define	MINVISBLKSIZ		    512
# define	STDVISBLKSIZ		   1024
# define	MAXVISBLKSIZ		1048576		/* 16 MiB */

# define	MINSTRIPECNT		      3
# define	MAXSTRIPECNT		      8

# define	VBF_READOK		0x00000001
# define	VBF_SEEKOK		0x00000002
# define	VBF_WRITOK		0x00000004
# define	VBF_TESTOK		0x00000010
# define	VBF_JUMPOK		0x00000020
# define	VBF_PEEKOK		0x00000040
/*
# define	VBF_COMPOK		0x00000100
*/
# define	VBF_RIFFOK		0x00000200
# define	VBF_ENDING		0x00001000
# define	VBF_SCRAPS		0x00002000
# define	VBF_USEFUL		0x00004000

# define	DFLAVROWS				16
# define	DFLAVCOLS				64

# define	BW_LIN		1
# define	BW_COL		1
# define	PW_LIN		1
# define	PW_WID		12
# define	SW_COL		1
# define	SW_HEI		4
# define	XW_HEI		(sectflag?4:6)
# define	XW_WID		PW_WID /* 12 */
# define	IW_HEI		SW_HEI /* 4 */
# define	IW_WID		PW_WID /* 12 */

# define	BW_HEI		2+avrows
# define	BW_WID		2+avcols
# define	PW_COL		wscols-13
# define	PW_HEI		12 /* 2+avrows */
# define	SW_LIN		wsrows-5
# define	SW_WID		BW_WID /* wscols-2 */
# define	XW_LIN		((BW_HEI)-(sectflag?3:5)) /* 1+(PW_HEI) */
# define	XW_COL		PW_COL
# define	IW_LIN		SW_LIN
# define	IW_COL		PW_COL

# define	NEXTDECRADDR(X,T)		( (T-1) - X )
# define	NEXTSCATADDR(X,T)		( (T&1) ? ( (X&1) ? (T/2)+(X/2)+1 : (T/2)-(X/2) ) : ( (X&1) ? (T/2)+(X/2) : (T/2)-(X/2)-1 ) )
# define	NEXTCONVADDR(X,T)		( (X&1) ? (X/2) : (T-1)-(X/2) )
# define	NEXTSPOTADDR(X,T)		( (X&1) ? ( T&1 ? X : X-1 ) : (T-1)-X )
# define	NEXTGOLDADDR(X,T)		( ( X < T/2 ) ? ( ( T & 1 ) ? ( (X&1) ? (T/2)+X+1 : (T/2)-X ) : ( (X&1) ? (T/2)+X : (T/2)-X-1 ) ) : ( (X&1) ? (T/2)+(T-X)-1 : X-(T/2)-(T&1) ) )
# define	NEXTRANDADDR(X,T)		( vdtrand () % T )

#ifdef SLICADDR
# define	NEXTSERISLICADDR(X,T)	( nextserislicaddr (X,T) )
# define	NEXTDISTSLICADDR(X,T)	( nextdistslicaddr (X,T) )
#endif

# define	VDT_INCR	'I'
# define	VDT_DECR	'D'
# define	VDT_SCAT	'S'
# define	VDT_CONV	'C'
# define	VDT_SPOT	'X'
# define	VDT_GOLD	'G'
# define	VDT_RAND	'R'

/*	__	__	__	__	__	__	__	__	*/

struct visblkdat {
	long	vbd_size ;
	off_t	vbd_addr ;
	int		vbd_lin ;
	int		vbd_col ;
	int		vbd_hits ;
	long	vbd_flags ;
} ;

typedef		struct visblkdat	VISBLKDAT ;

struct vdtperf {
	double		vdp_min ;		/* min throughput (bytes/usec)	*/
	double		vdp_max ;		/* max throughput (bytes/usec)	*/
	double		vdp_avg ;		/* avg throughput (bytes/usec)	*/
	double		vdp_sum ;
	double		vdp_siz ;		/* total size accounted for		*/
} ;

typedef		struct vdtperf		VDTPERF ;

struct vdtmark {
	char *		vdm_nam ;
	long long	vdm_min ;		/* min i/o time (usecs)			*/
	long long	vdm_max ;		/* max i/o time (usecs)			*/
	long long	vdm_avg ;		/* avg i/o time (usecs)			*/
	long long	vdm_sum ;
	long		vdm_cnt ;
	VDTPERF		vdm_prf ;
} ;

typedef		struct vdtmark		VDTMARK ;

# define	VISBLKDATSIZ		( sizeof (VISBLKDAT) )

struct vdtypeinfo {
	int *	vty_flag ;
	char *	vty_name ;
} ;

typedef		struct vdtypeinfo		VDTYPEINFO ;

/*	__	__	__	__	__	__	__	__	*/

# define			SPECTABXOPEN	 0
# define			SPECTABXCLOS	 1
# define			SPECTABXREAD	 2	/* read from 1st file			*/
# define			SPECTABXSEEK	 3	/* seek to read from 1st file	*/
# define			SPECTABXSTAT	 4
# define			SPECTABXWRIT	 5	/* write on second file			*/
# define			SPECTABXBLOT	 6
# define			SPECTABXCOOL	 7
# define			SPECTABXCOPY	 8
/*
# define			SPECTABXMESS	 9
*/
# define			SPECTABXAVER	10
# define			SPECTABXCOMP	11
# define			SPECTABXJUMP	12	/* seek to write on 2nd file	*/
# define			SPECTABXMAKE	13
# define			SPECTABXRIFF	14	/* read from 2nd file			*/
# define			SPECTABXPEEK	15	/* seek to read from 2nd file	*/
# define			SPECTABXSTIR	16	/* read, write someplace else	*/
# define			SPECTABXPOKE	16	/* write single sector			*/

/*	__	__	__	__	__	__	__	__	*/

void vdtreport () ;
void vdtinit () ;
void vdt () ;

#ifdef PROTOIOCTL
# ifdef ANSI
int ioctl(int d, int request, ...);
# else
int ioctl();
# endif
#endif

/*	__	__	__	__	__	__	__	__	*/

# endif /* ! _VDT_H_ */

/*
 * vi:nu ts=4
 */
